@@ -229,6 +229,7 @@ Api::end_rule ()
 	r->end = r->nbody+n;
 	r->pend = r->end;
 	program->size_of_program += n+1;
+	r->atleast = atleast_body;
 	r->lit = atleast_body;
 	r->inactive = atleast_body - n;
 	for (i = 0; i < size (nbody); i++)
@@ -261,6 +262,7 @@ Api::end_rule ()
 	r->hend = r->head+heads;
 	r->pbody = r->hend;
 	program->size_of_program += 2*heads;   // heads + nbody
+	r->atleast = atleast_head;
 	r->neg = heads - atleast_head;
 	r->inactiveNeg = -atleast_head;
 	for (i = 0; i < size (head); i++)
@@ -528,7 +530,7 @@ Api::copy (Api *api)
 	      add_body ((*a)->copy, false);
 	    for (Atom **a = r->pbody; a != r->pend; a++)
 	      add_body ((*a)->copy, true);
-	    set_atleast_body (r->atleast ());
+	    set_atleast_body (r->atleast);
 	    end_rule ();
 	    break;
 	  }
@@ -540,7 +542,7 @@ Api::copy (Api *api)
 	      add_head ((*a)->copy);
 	    for (Atom **a = r->pbody; a != r->pend; a++)
 	      add_body ((*a)->copy, true);
-	    set_atleast_head (r->atleast ());
+	    set_atleast_head (r->atleast);
 	    end_rule ();
 	    break;
 	  }
@@ -584,8 +586,6 @@ Api::copy (Api *api)
 	  break;
 	}
     }
-  for (Node *n = api->program->atoms.head (); n; n = n->next)
-    n->atom->source = 0;
 }
 
 void
@@ -595,6 +595,7 @@ Api::done ()
   for (Node *n = program->atoms.head (); n; n = n->next)
     {
       Atom *a = n->atom;
+      a->source = 0;     // Not null if copied
       if (a->headof + a->posScore + a->negScore)
 	{
 	  a->head = new Follows[a->headof + a->posScore + a->negScore];
